@@ -51,7 +51,6 @@
  */
 
 
-/* All drivers should typically include these */
 #include "xf86.h"
 #include "xf86_OSproc.h"
 #include "xf86Resources.h"
@@ -2048,6 +2047,8 @@ MGAPreInit(ScrnInfoPtr pScrn, int flags)
         MGAUnmapMem(pScrn);    
         pMga->pMgaHwInfo = (LPMGAHWINFO) xalloc (sizeof(MGAHWINFO));
         MGAGetHardwareInfo(pMga->pBoard,pMga->pMgaHwInfo);
+
+        
         /* copy the board handles */
         if(xf86IsEntityShared(pScrn->entityList[0])) {
 	    pMgaEnt->pClientStruct = pMga->pClientStruct;
@@ -2085,8 +2086,7 @@ MGAPreInit(ScrnInfoPtr pScrn, int flags)
         pMga->pMgaHwInfo = pMga->pMgaHwInfo;
     }
     if(MGAValidateMode(pMga->pBoard,&mgaModeInfo) != 0) {
-	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-		   "MGAValidateMode found the mode to be invalid\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "MGAValidateMode from HALlib found the mode to be invalid\n");
         return FALSE;
     }
 
@@ -2555,15 +2555,25 @@ MGAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 #ifdef USEMGAHAL
     FillModeInfoStruct(pScrn,mode);
 
-    if(pMga->SecondCrtc == TRUE) {
+    if(pMga->SecondCrtc == TRUE)
+    {
 
-    if (digital == TRUE)      
+        if (digital == TRUE)      
         {
             pMga->pMgaModeInfo->flOutput = MGAMODEINFO_DIGITAL2 |
                 MGAMODEINFO_SECOND_CRTC |
                 MGAMODEINFO_FORCE_PITCH |
                 MGAMODEINFO_FORCE_DISPLAYORG;
         }
+        else if (tv == TRUE)
+        {
+            pMga->pMgaModeInfo->ulRefreshRate = 60;
+            pMga->pMgaModeInfo->flSignalMode = 0x10;
+            pMga->pMgaModeInfo->flOutput = MGAMODEINFO_TV |
+                MGAMODEINFO_SECOND_CRTC |
+                MGAMODEINFO_FORCE_PITCH |
+                MGAMODEINFO_FORCE_DISPLAYORG;
+        }
         else 
         {
             pMga->pMgaModeInfo->flOutput = MGAMODEINFO_ANALOG2 |
@@ -2571,26 +2581,28 @@ MGAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
                 MGAMODEINFO_FORCE_PITCH |
                 MGAMODEINFO_FORCE_DISPLAYORG;
         }
+        
 
-    } else {
+    }
+    else
+    {
 	pMga->pMgaModeInfo->flOutput = MGAMODEINFO_ANALOG1 | 
-					MGAMODEINFO_FORCE_PITCH;
-        /*Clone */
-        if (digital == TRUE) pMga->pMgaModeInfo->flOutput |= MGAMODEINFO_DIGITAL2;
+                                       MGAMODEINFO_FORCE_PITCH;
     }
 
     pMga->pMgaModeInfo->ulFBPitch = pScrn->displayWidth;
+    /*Validate the parameters*/
     if(MGAValidateMode(pMga->pBoard,pMga->pMgaModeInfo) != 0) {
-	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,"Invalid parameters\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "MGAValidateMode from HALlib found the mode to be invalid\n");
 	return FALSE;
     }
-
+    
+    /*Validates the Video parameters*/
     if(MGAValidateVideoParameters(pMga->pBoard,pMga->pMgaModeInfo) != 0) {
-	xf86DrvMsg(pScrn->scrnIndex, X_ERROR,"Invalid parameters\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "MGAValidateVideoParameters from HALlib found the mode to be invalid\n");
 	return FALSE;
     }
 #endif /* USEMGAHAL */
-
 #ifdef XF86DRI
    if (pMga->directRenderingEnabled) {
        DRILock(screenInfo.screens[pScrn->scrnIndex], 0);
@@ -2598,20 +2610,13 @@ MGAModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 #endif
 
 #ifdef USEMGAHAL
-   if (tv == TRUE)
-   {
-       if(MGASetTVMode(pMga->pBoard,pMga->pMgaModeInfo) != 0)
+   /*Initialize the board*/
+   if(MGASetMode(pMga->pBoard,pMga->pMgaModeInfo) != 0)
        {
+           xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "MGASetMode returned an error. Make sure to validate the mode before.\n");
            return FALSE;
        }
-   }
-   else
-   {
-       if(MGASetMode(pMga->pBoard,pMga->pMgaModeInfo) != 0)
-       {
-           return FALSE;
-       }
-   }
+
 
    
 #define outMGAdreg(reg, val) OUTREG8(RAMDAC_OFFSET + (reg), val)
@@ -2755,6 +2760,19 @@ MGAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	  
 	  pMga->pMgaHwInfo = (LPMGAHWINFO) xalloc (sizeof(MGAHWINFO));
 	  MGAGetHardwareInfo(pMga->pBoard,pMga->pMgaHwInfo);
+
+          /*Detecting for type of display */
+          if (pMga->pMgaHwInfo->ulCapsSecondOutput & MGAHWINFOCAPS_OUTPUT_TV )
+          {
+              xf86DrvMsg(pScrn->scrnIndex, X_INFO, "TV detected\n ");
+          }
+          if (pMga->pMgaHwInfo->ulCapsSecondOutput & MGAHWINFOCAPS_OUTPUT_DIGITAL)
+          {
+              xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Digital Screen  detected\n ");
+          }
+
+
+                        
 	  /* Now copy these to the entitystructure */
 	  pMgaEnt->pClientStruct = pMga->pClientStruct;
 	  pMgaEnt->pBoard = pMga->pBoard;
